<?php

/*
 * Copyright (C) Hawks Solutions  - All Rights Reserved 
 * 
 * Unauthorized copying of this file, via any medium is strictly prohibited 
 * 
 * Proprietary and confidential 
 * 
 * Written by Anuraj <anurajpr2266@gmail.com>, September 2017  
 */

/*
 * Common Model , Define common functions for the system
 */

class Common extends MY_Model {
    /*
     * function validateDate ()
     * 
     * @param string
     * 
     * @return boolean
     */

    function validateDate($date = null) {
        if (!empty($date)) {
            $d = DateTime::createFromFormat('Y-m-d', $date);
            return $d && $d->format('Y-m-d') === $date;
        }
        return false;
    }

    /*
     * fn to get contents of a particular content type.
     * param is content type id.
     * returns result set.
     */

    function getContent($typeId) {
        $query = $this->db->select('id,LEFT(content,50) as content,content_type_id,created_date,created_by')
                ->from('content')
                ->where('content_type_id', $typeId)
                ->get();
        if (!empty($query)) {
            return $query->result();
        } else {
            return null;
        }
    }
    
    /*
     * function getAllContentTypes()
     * 
     * @param string
     * 
     * @return json
     */

    function getAllContentTypes($where = null) {
        $query = null;
        if (empty($where)) {
            $query = $this->db->query('SELECT * FROM content_types');
        } else {
            $query = $this->db->query('SELECT * FROM content_types WHERE ' . $where);
        }

        return json_encode($query->result_array());
    }

    /*
     * function getContentByType()
     * 
     * @param $contenttypeid
     * 
     * @return json
     */

    function getContentByType($contenttype = null) {

        if (!empty($contenttype)) {
            $query = $this->db->query('SELECT c.id as contentId,c.content,c.content2,c.end_date,c.name,c.created_date,c.created_by,ct.* FROM content c,content_types ct WHERE c.content_type_id='.$contenttype.' and c.content_type_id=ct.id;');
            return json_encode($query->result_array());
        }

        return json_encode(array());
    }
    /*
     * function getContentForAssignByType()
     * 
     * @param $contenttypeid
     * 
     * @return json
     */

    function getContentForAssignByType($contenttype = null) {

        if (!empty($contenttype)) {
           $query = $this->db->select('id,name')
                ->from('content')
                ->where('content_type_id', $contenttype)
                ->order_by('id','desc')
                ->get();
        if (!empty($query)) {
            return $query->result();
        } else {
            return null;
        }
    }
}
     /*
     * function getSessionForBookingById()
     * 
     * @param $contenttypeid
     * 
     * @return json
     */

    function getSessionForBookingById($contenttype = null) {

        if (!empty($contenttype)) {
           $query = $this->db->select('*')
                ->from('sessions')
                ->where('id', $contenttype)
                // ->order_by('id','desc')
                ->get();
        if (!empty($query)) {
            return $query->result();
        } else {
            return null;
        }
    }
}
    
    /*
     * fn to get images of a particular content type.
     * param is content type id.
     */
    function getImages($typeId) {
        $query = $this->db->select('contentId,path,thumb_path,caption,status')
                ->from('uploads')
                ->where('contentTypeId', $typeId)
                ->where('status','active')
                ->order_by('id','desc')
                ->get();
        if (!empty($query)) {
            return $query->result();
        } else {
            return null;
        }
    }
    
    /*
     * fn to get contents of a particular content type.
     * param is content type id.
     * returns result set.
     */

    function getFullContent($typeId) {
        $query = $this->db->select('c.id,c.name,c.content,c.content2,c.content_type_id,c.created_date,c.created_by,u.path,c.end_date,u.status,u.caption')
                ->from('content c')
             
                ->join('uploads u','u.contentId=c.id','LEFT')
                ->where('c.content_type_id', $typeId)
                  ->order_by('id','desc')
                     ->distinct()
                ->where('u.status', 'active')
                ->get();
        if (!empty($query)) {
            return $query->result_array();
        } else {
            return null;
        }
    }
    function getOldpassword($password){
        $paswrd=$this->db->select('id')
                ->from('auth')
                ->where('password',$password)
                ->get()->result();
if(!empty($paswrd)) {
            return $paswrd;
        } else {
            return null;
        }
    }
    
    
    function savenewpassword($paswrd,$id){
        $data = array('password' => $paswrd);
        $this->db->where('id', $id)
                ->update('auth', $data);
        return true;
    }
    
      /*
     * fn to delete 
     * param are tablename, primary key value,
     */

    function softdelete($table = null, $key = null,$value = null) {
            $query = DB::table($table)
                        ->when($value!=null, function($query) use ($key,$value) {
                           return  $query->where($key,$value); 
                        })
                        ->when($value==null, function($query) use ($key) {
                           return  $query->where('id',$key); 
                        })
                    ->update(['deleteStatus' =>0]);
            if (!empty($query)) {
                return $query;
            }
       
        return false;
    }
    function insertLogs($data){
        $logdata=array( 'username'=>$data['username'],'contentId'=>$data['contentId'],'description'=>$data['description'],'imgId'=>$data['imgId']);
        $this->db->insert('logs', $logdata);
    }
  /*
     * fn uploadImage()
     * fn to upload the image
     * param are image and folder name.
     */

    public function uploadImage($files, $images=null) {
 
        $image = $files['imageUrl']['name'];
        // var_dump($image);
        if (!file_exists(base_url("assets/images/uploads/"))) {
            mkdir(base_url("assets/images/uploads/"), 0777, true);
        }
        $imageName = date('Y-m-d H:i:s') . '-' . $image;
        
         $config['upload_path'] = './assets/images/uploads/';
        $config['allowed_types'] = 'gif|jpg|png|pdf|jpeg';
        $config['max_size'] = 4248;
        $config['max_width'] = 0;
        $config['max_height'] = 0;
        $new_name = time();
        $config['file_name'] = $new_name;
    
        $this->load->library('upload', $config);
        if (!$this->upload->do_upload('imageUrl')) {
            $data['imageError'] = $this->upload->display_errors();
            $sourceImg = "";
            return $data;
        } else {
             $data = array('upload_data' => $this->upload->data());            
            $data['upload_data']['path'] = "assets/images/uploads/" . $data['upload_data']['file_name'];
            ///////////////////////Image resize///////////////////////////////////////
            $config['image_library'] = 'gd2';
            $config['source_image'] = './assets/images/uploads/' . $data['upload_data']['file_name']; //$data['upload_data']['full_path'];
            $config['create_thumb'] = TRUE;
//            $config['maintain_ratio'] = TRUE;
            $config['width'] = 240;
            $config['height'] = 161;
            $config['new_image'] = './assets/images/uploads/thumbnails/' . $data['upload_data']['file_name'];
            $this->load->library('image_lib', $config);
            $this->image_lib->initialize($config);
          //  if($data['upload_data']['file_ext']!=".pdf"){
            // if (!$this->image_lib->resize()) {
            //     echo $this->image_lib->display_errors();
            //     die;
            // }
          //  }
            
            $data['upload_data']['thumb_path'] = 'assets/images/uploads/thumbnails/' . $data['upload_data']['raw_name'] . '_thumb' . $data['upload_data']['file_ext'];
          
            $data = $this->upload->data();
            // var_dump($data['full_path']);
            $sourceImg =  $data['full_path'];
            // $partialName = "assets/$images/".$data['file_name'];
//            $thumbGeneration = $this->thumb($data, 200, true);
//            if ($thumbGeneration) {
//                $explode = explode('.', $sourceImg);
//                $thumbImg = $explode[0] . '_thumb.' . $explode[1];
//            } else {
//                $thumbImg = "";
//            }
        }
        if ($sourceImg) {
            $imageArray = array(
               'path' => $sourceImg
           
            );
            return $imageArray;
        }
    }
    
     /*
    * Fun to send Mail
    */
        public function sendMail($to,$subject,$body,$attachments=null){
// var_dump($attachments);
        $config = Array(
                     
                        'smtp_host' => 'mail.hawkssolutions.com',
                        'smtp_port' => 465,
                        'smtp_user' => 'nighil@hawkssolutions.com',
                        'smtp_pass' => '*hawks#hst*',
                        'mailtype'  => 'html', 
                        'charset'   => 'iso-8859-1',
                        'encryption'  =>'ssl',  
                        );


              $this->load->library('email', $config);
              $this->email->clear();
              $this->email->set_newline("\r\n");
              $this->email->from('info@oxoconsultants.com');
              $this->email->to($to);
              $this->email->subject($subject);
              $this->email->from('info@hawkssolutions.com');
              $this->email->message($body); 
              if(!empty($attachments)){
               $attched_file=$attachments; 
               $this->email->attach($attched_file);
              }

               if ( ! $this->email->send()) {
        show_error($this->email->print_debugger());
    } else {
            echo('DONE');        
        } 
            // $headers  = "From: Contact Form <info@hawkssolutions.com>\n";
            // $headers .= "Cc: Contact Form < nighil05@gmail.com >\n"; 
            // $headers .= "X-Sender: testsite <info@hawkssolutions.com>\n";
            // $headers .= 'X-Mailer: PHP/' . phpversion();
            // $headers .= "X-Priority: 1\n"; // Urgent message!
            // $headers .= "Return-Path: info@hawkssolutions.com\n"; // Return path for errors
            // $headers .= "MIME-Version: 1.0\r\n";
            // $headers .= "Content-Type: text/html; charset=iso-8859-1\n";
            // if(mail($to,$subject,$body,$headers)){
              //    echo "Mail Send";
              //   return true;
              // }else{
              //   $this->email->get_debugger_messages();
              //   return false;
              // }
     }
    
    
    
    function getallfromlogs(){
        $logs=$this->db->select('*')
                ->from('logs')
                ->order_by('id','DESC')
                ->get()->result();
        return $logs;
    }
   
    public function commonMessage($type = null) {
        if (!empty($type)) {
            switch ($type) {
                case SUCCESSMAIL:
                    $this->session->set_flashdata('message', array('type' => 'success', 'message' => 'Mail Send Successfully'));
                    break;
                case ERRORMAIL:
                    $this->session->set_flashdata('message', array('type' => 'error', 'message' => 'Unable To Mail'));
                    break;
                default:
                    $this->session->set_flashdata('message', array('type' => 'warning', 'message' => 'Something went abnormal'));
                    break;
            }
        }
    }

}
